package Treaty;
#
use strict;
use vars  qw/$VERSION $AUTHOR/;
use POSIX qw/isdigit/;
use GPG;

my %error_code =
        ('CREDITOR_ID_EMPTY'    => 'Creditor ID is empty',
         'DEBITOR_ID_EMPTY'     => 'Debitor ID is empty',
         'AMOUNT_EMPTY'         => 'Amount is empty',
         'CURRENCY_EMPTY'       => 'Currency is empty',
         'CREDITOR_BAD_FORMAT'  => 'Creditor contains illegal character ":"',
         'DEBITOR_BAD_FORMAT'   => 'Debitor contains illegal character ":"',
         'AMOUNT_BAD_FORMAT'    => 'Amount is not a positive number ":"',
         'CURRENCY_BAD_FORMAT'  => 'Currency contains illegal character ":"',
         'PROMISE_EMPTY'        => 'This treaty does not contain any promise',
         'NO_MATCHING_ID'       => 'Neither the creditor nor the debitor correspond to the ID',
         'ALLREADY_SIGNED'      => 'The treaty is allready signed',
         'GPG_ERROR'            => 'The following GPG error occured:');

$VERSION = "0.02";
$AUTHOR  ="Tomas Pospisek <tpo\@spin.ch>";

### public methods ##############################################

### new #########################################################
#
# can be called in the following two forms:
#
#   new Treaty($id_string_creditor, $id_string_debitor, $amount, $currency)
#
# with all parameters be set or:
#
#   new Treaty()
#
################################################################

sub new ($%) {
  my ($this, @contents) = @_;
  my $class = ref($this) || $this;
  my $self  = {};
  
  undef $self->{'promise'};
  undef $self->{'signature_1'};
  undef $self->{'signature_2'};
  undef $self->{'error'};

  bless $self, $class;

  $self->fill_contents(@contents) if @_ > 1;

  return $self;

  print "blabla $self->$error_code{'CREDITOR_ID_EMPTY'}\n";
}

### error #########################################################

sub error {
  my ($self) = @_;
  return $self->{'error'};
}

### contents ######################################################

sub contents {
  my ($self,@contents) = @_;
  undef $self->{'error'};

  if (@_ > 1) {
    $self->fill_contents(@contents); }
  else {
    return $self->split_contents();; }
}
      
### promise #######################################################

sub promise {
  my ($self) = @_;
  undef $self->{'error'};

  if (not defined $self->{'promise'}) {
    $self->{'error'} = $error_code{'EMPTY_PROMISE'};
    return undef; }
  else {
    return $self->{'promise'};}
}

### sign ##########################################################

sub sign {
  my ($self, $gpg, $key_id, $passphrase) = @_;
  undef $self->{'error'};

  my ($id_creditor, $id_debitor, $amount, $currency) =
    $self->split_contents();
  return undef if defined $self->{'error'};

  if (     not $self->is_key_owner($gpg, $key_id, $id_creditor)
       and not $self->is_key_owner($gpg, $key_id, $id_debitor)) {
    $self->{'error'} = $error_code{'NO_MATCHING_ID'};
    return undef; }

  if ( not defined $self->promise()) { # self->promise() will set error
    return undef; }

  if ( defined $self->{'signature_1'} and defined $self->{'signature_2'}) {
    $self->{'error'} = $error_code{'ALLREADY_SIGNED'};
    return undef; }

  my $signature = $gpg->clearsign($key_id, $passphrase, $self->promise());
  if ($gpg->error()) {
    $self->{'error'} = $error_code{'GPG_ERROR'}.$gpg->error();
    return undef;}
    
  if (defined $self->{'signature_1'}) {
    $self->{'signature_2'} = $signature; }
  else {
    $self->{'signature_1'} = $signature;
  }
}

### is_key_owner ####################################################

sub is_key_owner {
  my ($self, $gpg, $key_id, $id_string) = @_;

  my $reference_chaos = $gpg->list_keys();

  my $i=0;
  while($reference_chaos->[$i]) {
    if(     $reference_chaos->[$i]{'key_id'}  eq $key_id
        and $reference_chaos->[$i]{'user_id'} eq $id_string) {
      return 1;
    }
    $i++;
  }
  return 0;
}

### private methods ###############################################

### fill_contents #################################################

sub fill_contents{
  my ($self, $id_string_creditor,
             $id_string_debitor,
             $amount,
             $currency) = @_;
  undef $self->{'error'};

  $self->{'promise'} = '';

  $self->{'error'} = $error_code{'CURRENCY_BAD_FORMAT'} and return $self
    if not is_right_format($currency);


  if (not defined $id_string_creditor) {
    $self->{'error'} = $error_code{'CREDITOR_ID_EMPTY'};
    return $self;}
  elsif (not is_right_format($id_string_creditor)) {
    $self->{'error'} = $error_code{'CREDITOR_BAD_FORMAT'};
    return $self; }
  else {
    $self->{'promise'} = "CREDITOR : $id_string_creditor"; }

  if (not defined $id_string_debitor) {
    $self->{'error'} = $error_code{'DEBITOR_ID_EMPTY'};
    return $self;}
  elsif (not is_right_format($id_string_debitor)) {
    $self->{'error'} = $error_code{'DEBITOR_BAD_FORMAT'};
    return $self; }
  else {
    $self->{'promise'} .= " : DEBITOR : $id_string_debitor"; }

  if (not defined $amount) {
    $self->{'error'} = $error_code{'AMOUNT_ID_EMPTY'};
    return $self;}
  elsif (not isdigit($amount) or $amount < 0) {
    $self->{'error'} = $error_code{'AMOUNT_BAD_FORMAT'};
    return $self; }
  else {
    $self->{'promise'} .= " : AMOUNT : $amount"; }

  if (not defined $currency) {
    $self->{'error'} = $error_code{'CURRENCY_ID_EMPTY'};
    return $self;}
  elsif (not is_right_format($currency)) {
    $self->{'error'} = $error_code{'CURRENCY_BAD_FORMAT'};
    return $self; }
  else {
    $self->{'promise'} .= " : CURRENCY : $currency"; }

  return $self;
}
      
### split_contents ################################################

sub split_contents{
  my ($self) = @_;
  undef $self->{'error'};

  if (not defined $self->{'promise'}) {
    $self->{'error'} = $error_code{'EMPTY_PROMISE'};
    return $self; }
  else {
    my $x; # descriptor
    my ( $cr, $de, $am, $cu); #data
    ( $x, $cr, $x, $de, $x, $am, $x, $cu) = split(/ : /, $self->{'promise'});
    return ($cr, $de, $am, $cu); }
}

### is_right_format ###############################################

sub is_right_format{
  my ($string) = @_;
  return 0 if $string =~ /:/;
  return 1;
}

1; # END
